library(shiny)
library(ggplot2)
library(sf)
library(cowplot)
library(dplyr)

#Data generated by https://github.com/VictimOfMaths/Experiments/blob/master/COVID19BivariateMaps.R
#Written out and then reduced in size through https://mapshaper.org/

map.data <- st_read("data/COVID19LSOA.shp")

#Generate key
keydata <- data.frame(IMDtert=c(1,1,1,2,2,2,3,3,3), morttert=c(1,2,3,1,2,3,1,2,3),
                      RGB=c("#e8e8e8","#ace4e4","#5ac8c8","#dfb0d6","#a5add3",
                            "#5698b9","#be64ac","#8c62aa","#3b4994"))
key <- ggplot(keydata)+
  geom_tile(aes(x=morttert, y=IMDtert, fill=RGB))+
  scale_fill_identity()+
  labs(x = expression("Greater age-based COVID-19 risk" %->%  ""),
       y = expression("Greter health deprivation" %->%  "")) +
  theme_classic() +
  # make font small enough
  theme(
    axis.title = element_text(size = 8),axis.line=element_blank(), 
    axis.ticks=element_blank(), axis.text=element_blank())+
  # quadratic tiles
  coord_fixed()

server <- function(input, output) {
  
  output$plot <- renderPlot({
    
    #Stop plot updating without clicking button
    input$run
    
    isolate({
      
      #Mess about with user LA input to handle multiple LAs
      #First split into LA-specific chunks
      int <- as.data.frame(strsplit(input$userLA, ":", fixed=TRUE))
      #Then strip off leading and trailing spaces
      int <- as.data.frame(trimws(int[,c(1)], which="both"))
      
      #Draw main plot
      plot <- ggplot(subset(map.data, LAname %in% int[,c(1)]), aes(fill=RGB, geometry=geometry))+
        geom_sf(colour=ifelse(input$LSOABoundaries==FALSE ,NA, "Gray30"))+
        theme_classic()+
        scale_fill_identity()+
        theme(axis.line=element_blank(), axis.ticks=element_blank(), axis.text=element_blank(),
              axis.title=element_blank(),  plot.title=element_text(face="bold"))+
        labs(title=paste("Mapping potential COVID-19 risk across", input$userLAname),
             subtitle="LSOA-level health deprivation and potential COVID-19 mortality risk based on age-sex structure of population",
             caption="Population data from ONS, CFRs adapted from ISS & Imperial data\nPlot by @VictimOfMaths")
      
      #Translate legend position selection into plot position
      legpos <- case_when(
        input$legendpos==1 ~ "0.03,0.6,0.3,0.3",
        input$legendpos==2 ~ "0.68,0.6,0.3,0.3",
        input$legendpos==3 ~ "0.03,0.03,0.3,0.3",
        input$legendpos==4 ~ "0.68,0.05,0.3,0.3",
        input$legendpos==5 ~ "0,0,0,0"
      )
      
      #Stick plot and legend together
      finalplot <- ggdraw()+
        draw_plot(plot, 0,0,1,1)+
        eval(parse(text=ifelse(input$legendpos==5, "NULL", paste0("draw_plot(key,", legpos, ")"))))
      
      covid_download<<-reactive({finalplot})
      
      print(finalplot)
      
    })
  })
  
  output$covid_download = downloadHandler(
    filename = "COVIDmap.png",
    content = function(file) {
      device <- function(..., width, height) {
        grDevices::png(..., width = 12, height = 8,
                       res = 300, units = "in")
      }
      ggsave(file, 
             plot = covid_download(), 
             device = device)
    })
}
